const system = require("../../../system")
const settings = require("../../../../config/settings")
const md5 = require("MD5");
class SignService{
  constructor() {
    this.centerAppUrl = settings.centerAppUrl();
    this.restClient = system.getObject("util.restClient");
    this.redisClient = system.getObject("util.redisClient");
  }
  async restPostUrl(pobj, url) {
    try {
      var rtn = await this.restClient.execPost(pobj, url);
      if (!rtn || !rtn.stdout) {
        return system.getResult(null, "restPost data is empty");
      }
      var result = JSON.parse(rtn.stdout);
      return result;
    } catch (e) {
      var errorMsg = "error：" + e.stack;
      return system.getResult(null, errorMsg);
    }
  }
  async getAppInfoByAppKey(appkey){
    try {
        var shaStr = "getAppInfoByAppKey_"+appkey;
        var cacheRes = await this.redisClient.get(shaStr);
        if (cacheRes) {
            return JSON.parse(cacheRes);
        }
        let url = this.centerAppUrl + 'payment/paymentApi/getAppInfoByAppKey';
        let appInfoRt = await this.restPostUrl({ appKey: appkey }, url);
        if(appInfoRt && appInfoRt.status==0){
            //保存缓存
            await this.redisClient.setWithEx(shaStr, JSON.stringify(appInfoRt), 3600);
        }
        return appInfoRt;
    } catch (error) {
        var errres = system.getResultFail("获取应用信息失败！");
        return errres;
    }
  }
  /**
    * 验证签名
    * @param {*} params 要验证的参数 
    * @param {*} app_secret 应用的校验key
    */
   async verifySign(params, app_secret) {
    if (!params) {
      return system.getResult(null, "请求参数为空");
    }
    if (!params.sign) {
      return system.getResult(null, "请求参数sign为空");
    }
    var signArr = [];
    var keys = Object.keys(params).sort();
    if (keys.length == 0) {
      return system.getResult(null, "请求参数信息为空");
    }
    for (let k = 0; k < keys.length; k++) {
      const tKey = keys[k];
      if (tKey != "sign" && params[tKey]) {
        let tmpKeyValue = params[tKey];
        if (tmpKeyValue instanceof Array || tmpKeyValue instanceof Object) {
          tmpKeyValue = JSON.stringify(tmpKeyValue);
        }
        signArr.push(tKey + "=" + tmpKeyValue);
      }
    }
    if (signArr.length == 0) {
      return system.getResult(null, "请求参数组装签名参数信息为空");
    }
    var resultSignStr = signArr.join("&") + "&key=" + app_secret;
    var resultTmpSign = md5(resultSignStr).toUpperCase();
    if (params.sign != resultTmpSign) {
      return system.getResult(null, "返回值签名验证失败");
    }
    return system.getResultSuccess();
   }
   async createSign(params, app_key) {
    if (!params) {
      return system.getResultFail(-310, "请求参数为空");
    }
    var signArr = [];
    var keys = Object.keys(params).sort();
    if (keys.length == 0) {
      return system.getResultFail(-330, "请求参数信息为空");
    }
    for (let k = 0; k < keys.length; k++) {
      const tKey = keys[k];
      if (tKey != "sign" && params[tKey] && !(params[tKey] instanceof Array)) {
        signArr.push(tKey + "=" + params[tKey]);
      }
    }
    if (signArr.length == 0) {
      return system.getResultFail(-350, "请求参数组装签名参数信息为空");
    }
    var resultSignStr = signArr.join("&") + "&key=" + app_key;
    var resultTmpSign = md5(resultSignStr).toUpperCase();
    return system.getResultSuccess(resultTmpSign);
  }

}
module.exports = SignService;
// var task = new SignService();
// var obj = {
//   "intentionBizId": "TRE-nmnHtEoTjeEtest",
//   "phone": "13126673246",
//   "userName": "",
//   "description": "111111",
//   "area": "BEIJING",
//   "consultType": "EDI"
// };
// task.createSign(obj,"7cbb892450174167b5c7e01we4716t51").then(d=>{
//     console.log(d,"!!!!!!!!!!!!!!!!!!!!!!!!!!");
// });
